/*
 * sl_protocol.h
 *
 *  Created on: 2024620
 *      Author: psx
 */

#ifndef APP_INC_SL_PROTOCOL_H_
#define APP_INC_SL_PROTOCOL_H_

#include "debug.h"
#include "uart_dev.h"
#include "math.h"

/*  */
typedef enum
{
   SL_Function_Code_Read_Register           =   0x30,   /* Ĵ */
   SL_Function_Code_Write_Register          =   0x10,   /* дĴ */
   SL_Function_Code_Broadcast_Scan          =   0xA0,   /* 㲥ɨ */
   SL_Function_Code_Registration_request    =   0xA1,   /* ע */
   SL_Function_Code_Update_Profile          =   0xD0,   /* ļ */
   SL_Function_Code_Remote_Upgrade          =   0xD1,   /* Զ */
}SL_MsgFunctionCode;

/* Ĵַ */
typedef enum
{
   SL_Register_Registration_Status          = 0x0000,  /* ע״̬  */
   SL_Register_address                      = 0x0001,  /* ַ  */
   SL_Register_Access_Node_Type             = 0x0002,  /* ڵ   */
   SL_Register_Communication_Methods        = 0x0003,  /* ͨŷʽ   */
   SL_Register_Output_Voltage               = 0x0100,  /* ѹ   */
   SL_Register_Battery_Voltage              = 0x0101,  /* صѹ   */
   SL_Register_Charg_Current                = 0x0102,  /* +أ   */
   SL_Register_Discharg_Current             = 0x0103,  /* ŵأ   */
   SL_Register_Solar_Open_Circuit_Voltage   = 0x0104,  /* ̫ܿ·ѹ */
   SL_Register_HighSideMos_Temperature      = 0x0105,  /* ߶mos¶ */
   SL_Register_DischargMos_State            = 0x0106,  /* ŵmos״̬ */
   SL_Register_MPPT_Mode                    = 0x0107,  /* ģʽ */
}SL_Mppt_MsgRegister;

/* ע״̬ */
typedef enum
{
   UNREGISTER = 0,          /* δע */
   REGISTER_FAIL = 1,       /* עʧ */
   REGISTER_SUCCESS = 2,    /* עɹ */
}SL_REGISTERSTATUS;

/* ڵ */
typedef enum
{
   POWERBOX = 1,            /* Դ */
   MICROMETEOROLOGY = 2,    /* ΢ */
}SL_ACCESSNODETYPE;

/* ͨŷʽ */
typedef enum
{
    RS485 = 1,
    RJ45 = 2,
    Lora = 3,
}SL_COMMUNICATIONMETHODS;

/* ŵmos״̬ */
typedef enum
{
    close = 0,
    open = 1,
}SL_DischargMos_State;

/* mpptģʽ */
typedef enum
{
    NoWork          = 0,    /* ûй */
    TRICKLE         = 1,    /* ģʽ */
    CONSTANTCURRENT = 2,    /* ģʽ */
    CONSTANTVOLTAGE = 3,    /* ѹģʽ */
    FLOAT           = 4,    /* ģʽ */
    NoBattery       = 5,    /* ûе */
}SL_MPPT_MODE;

#define chang_8_to_16(L,H)  (L | (H<<8))

/* ָ뷽ʽΪ1ֽ */
#pragma pack(push,1)
/* ʱյ֡ʽ->ڵ㣩 */
typedef struct _SL_Mppt_Rorecv_pack{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint8_t function_Code;                  /*  */
    uint8_t read_Register_Start_Address_H;  /* ȡĴʼַ8λ */
    uint8_t read_Register_Start_Address_L;  /* ȡĴʼַͰλ */
    uint8_t read_Register_Number_H;         /* ȡĴ߰λ */
    uint8_t read_Register_Number_L;         /* ȡĴͰλ */
    uint8_t check_Bit_H;                    /* Уλ߰λ */
    uint8_t check_Bit_L;                    /* УλͰλ */
    uint8_t end_Flag;                       /* ־ */
}SL_Mppt_Rorecv_pack;
#define SL_MPPT_RORECV_PACK_SIZE        (sizeof(SL_Mppt_Rorecv_pack))

/* дʱյ֡ʽ->ڵ㣩 */
typedef struct _SL_Mppt_Worecv_pack{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint8_t function_Code;                  /*  */
    uint8_t write_Register_Start_Address_H; /* дĴʼַ8λ  */
    uint8_t write_Register_Start_Address_L; /* дĴʼַͰλ  */
    uint8_t write_Register_Number_H;        /* дĴ߰λ  */
    uint8_t write_Register_Number_L;        /* дĴͰλ  */
    uint8_t *content;                       /* д */
    uint8_t check_Bit_H;                    /* Уλ߰λ */
    uint8_t check_Bit_L;                    /* УλͰλ */
    uint8_t end_Flag;                       /* ־ */
}SL_Mppt_Worecv_pack;
#define SL_MPPT_WORECV_PACK_SIZE        (sizeof(SL_Mppt_Worecv_pack))

/* ɨ㲥֡F1->ڵ㣩 */
typedef struct _SL_Mppt_ScanBroadcast_pack{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint8_t function_Code;                  /*  */
    uint8_t check_Bit_H;                    /* Уλ߰λ */
    uint8_t check_Bit_L;                    /* УλͰλ */
    uint8_t end_Flag;                       /* ־ */
}SL_Mppt_Scan_Broadcast_pack;
#define SL_MPPT_SCAN_BROADCAST_PACK_SIZE    (sizeof(SL_Mppt_Scan_Broadcast_pack))

/* ע֡F2ڵ->أ */
typedef struct _SL_Mppt_RegistrationRequest_pack{
    uint8_t start_Flag[2];              /* ʼ־  */
    uint8_t address[7];                 /* ַ */
    uint8_t function_Code;              /*  */
    uint8_t register_Length_H;          /* Ĵȸ߰λ */
    uint8_t register_Length_L;          /* ĴȵͰλ */
    uint8_t registration_Status_H;      /* ע״̬߰λ  */
    uint8_t registration_Status_L;      /* ע״̬Ͱλ */
    uint8_t access_Node_ID[7];          /* ڵID */
    uint8_t access_Node_Type_H;         /* ڵ͸߰λ */
    uint8_t access_Node_Type_L;         /* ڵ͵Ͱλ */
    uint8_t check_Bit_H;                /* Уλ߰λ */
    uint8_t check_Bit_L;                /* УλͰλ */
    uint8_t end_Flag;                   /* ־ */
}SL_Mppt_RegistrationRequest_pack;
#define SL_MPPT_REGISTRATIONREQUEST_PACK_SIZE    (sizeof(SL_Mppt_RegistrationRequest_pack))

/* עظ֡F3->ڵ㣩 */
typedef struct _SL_Mppt_RegistrationReply_pack{
    uint8_t start_Flag[2];              /* ʼ־  */
    uint8_t address[7];                 /* ַ */
    uint8_t function_Code;              /*  */
    uint8_t register_Length_H;          /* Ĵȸ߰λ */
    uint8_t register_Length_L;          /* ĴȵͰλ */
    uint8_t registration_Status_H;      /* ע״̬߰λ  */
    uint8_t registration_Status_L;      /* ע״̬Ͱλ */
    uint8_t check_Bit_H;                /* Уλ߰λ */
    uint8_t check_Bit_L;                /* УλͰλ */
    uint8_t end_Flag;                   /* ־ */
}SL_Mppt_RegistrationReply_pack;
#define SL_MPPT_REGISTRATIONREPLY_PACK_SIZE (sizeof(SL_Mppt_RegistrationReply_pack))

/* ֡ʽ->ڵ㣩 */
typedef struct _SL_Mppt_ROther_pack{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint8_t function_Code;                  /*  */
    uint8_t check_Bit_H;                    /* Уλ߰λ */
    uint8_t check_Bit_L;                    /* УλͰλ */
    uint8_t end_Flag;                       /* ־ */
}SL_Mppt_ROther_pack;
#define SL_MPPT_ROTHER_PACK_SIZE (sizeof(SL_Mppt_ROther_pack))

/* ֡ʽڵ->أ */
typedef struct _SL_Mppt_SOther_pack{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint8_t function_Code;                  /*  */
    uint8_t state;                          /* ׼״̬01H׼ */
    uint8_t check_Bit_H;                    /* Уλ߰λ */
    uint8_t check_Bit_L;                    /* УλͰλ */
    uint8_t end_Flag;                       /* ־ */
}SL_Mppt_SOther_pack;
#define SL_MPPT_SOTHER_PACK_SIZE (sizeof(SL_Mppt_SOther_pack))


/*  */
#define RECV_LENGTH 20
typedef struct _SL_Mppt_Recv_pack{
    uint8_t  start_Flag[2];                 /* ʼ־  */
    uint8_t  address[7];                    /* ַ */
    uint8_t  function_Code;                 /*  */
    uint8_t  recv_Data[RECV_LENGTH];        /*  */
}SL_Mppt_Recv_pack;

/* 봦 */
typedef void (*MsgProcFunc)(device_handle device, void*, uint32_t MsgLen);
typedef struct _SL_FunctionMsgProcTable{
    u_int32_t    msgId;
    MsgProcFunc  pMsgProc;
}SL_FuncionMsgProcTable;

/* Ĵ */
typedef uint16_t (*RegProcFunc)(void*);
typedef struct _SL_RegProcTable{
    uint32_t     regId;
    RegProcFunc  pRegProc;
}SL_RegProcTable;


/* Ĭϲ */
typedef struct _default_Value{
    uint8_t start_Flag[2];                  /* ʼ־  */
    uint8_t address[7];                     /* ַ */
    uint16_t access_Node_Type;              /* ڵ   */
    uint16_t communication_Methods;         /* ͨŷʽ */
    uint8_t end_Flag;                       /* ־ */
}default_Value;

extern default_Value defaultValue;

typedef struct _SL_Mppt_para{
  uint16_t Registration_Status;         /* ע״̬  */
  uint8_t  address[7];                  /* ַ  */
  uint16_t Access_Node_Type;            /* ڵ  */
  uint16_t Communication_Methods;       /* ͨŷʽ  */
  float_t Output_Voltage;               /* ѹ */
  float_t Battery_Voltage;              /* صѹ  */
  float_t Charg_Current;                /* +أ   */
  float_t Discharg_Current;             /* ŵأ   */
  float_t Input_Voltage;                /* ̫ܰѹ  */
  float_t Solar_Open_Circuit_Voltage;   /* ̫ܰ忪·ѹ  */
  float_t HighSideMos_Temperature;      /* ߶mos¶ */
  uint16_t DischargMos_State;           /* ŵmos״̬ */
  uint16_t MPPT_Mode;  /* ģʽ */
}SL_Mppt_para;
extern SL_Mppt_para g_Mppt_Para;

/* ָĬϵĶ */
#pragma pack(pop)

void read_and_process_uart_data(device_handle device);
uint16_t CheckFunc(uint8_t *arr_buff, uint8_t len);
int randomDelay();
uint8_t Check_485_bus_busy(device_handle device);



#endif /* APP_INC_SL_PROTOCOL_H_ */
