/*
 * uart_dev.c
 *
 *  Created on: 2024622
 *      Author: psx
 */

#include "uart_dev.h"
#include "inflash.h"

/* ʹ485 */
#define rs485_send_enable 1

static void uart_init(uartIndex_e uart_index, int baud);
static uint8_t uart_putchar(device_handle device, char ch);

device_handle g_bat485_uart3_handle;
device_handle g_gw485_uart4_handle;

static uint8_t bat485_in_buff[200];
static uint8_t gw485_in_buff[300];

uint8_t rs485_out_buff[100];


/**
  * @brief  Ϣʼںż.
  * @param  uart_index    ӦӲں
  * @param  uart_baudrate 
  */
uart_device_info uart_devices[]={
  [0] = {
      .init          = 0,
      .uart_index    = BAT485_UART_INDEX,
      .uart_baudrate = 9600,
  },
  [1] = {
      .init          = 0,
      .uart_index    = GW485_UART_INDEX,
      .uart_baudrate = 9600,
  },
};

/**
  * @brief  ʼ豸.
  * @param  uart_index ʼں
  * @param  buff       ѭbuffַ
  * @param  buff_size  ѭbuffӦС
  * @retval ھ
  */
//#define ELEMENT_OF(x) (sizeof(x) / sizeof((x)[0]))
//device_handle uart_dev_init(uartIndex_e uart_index, uint8_t *buff, int buff_size)
//{
//  int i = 0;
//  for(; i < ELEMENT_OF(uart_devices); i++){
//    if(uart_devices[i].uart_index == uart_index){
//      if(!uart_devices[i].init){
//        InitRingQueue(&uart_devices[i].uart_ring_queue, buff, buff_size);
//        uart_init(uart_index, uart_devices[i].uart_baudrate);
//
//        uart_devices[i].init = 1;
//      }
//      return (device_handle)(&uart_devices[i]);
//    }
//  }
//  return 0;
//}
device_handle uart_dev_init(void)
{
//    int i = 0;
//    for(; i < ELEMENT_OF(uart_devices); i++){
//      if(uart_devices[i].uart_index == uart_index){
//        if(!uart_devices[i].init){
//          InitRingQueue(&uart_devices[i].uart_ring_queue, buff, buff_size);
//          uart_init(uart_index, uart_devices[i].uart_baudrate);
//
//          uart_devices[i].init = 1;
//        }
//        return (device_handle)(&uart_devices[i]);
//      }
//    }
    InitRingQueue(&uart_devices[0].uart_ring_queue, bat485_in_buff, sizeof(bat485_in_buff));
    uart_init(BAT485_UART_INDEX, g_slConfigInfo.baud_485);
    uart_devices[0].init = 1;
    g_bat485_uart3_handle = (device_handle)(&uart_devices[0]);

    InitRingQueue(&uart_devices[1].uart_ring_queue, gw485_in_buff, sizeof(gw485_in_buff));
    uart_init(GW485_UART_INDEX, g_slConfigInfo.baud_485);
    uart_devices[1].init = 1;
    g_gw485_uart4_handle = (device_handle)(&uart_devices[1]);

    return 0;
}


/**
  * @brief  Ӳʼ.
  * @param  uart_index ں
  * @param  baud 
  * @retval None
  */
void uart_init(uartIndex_e uart_index, int baud)
{
    if (uart_index == BAT485_UART_INDEX) {
//        BAT_485_Init(uart_devices[0].uart_baudrate);
        BAT_485_Init(115200);
    } else if (uart_index == GW485_UART_INDEX) {
        GW_485_Init(uart_devices[1].uart_baudrate);
    }
}

/**
  * @brief  һֽ.
  * @param  uart_index ں
  * @param  ch ַ
  * @retval 1 ɹ  0ʧ
  */
uint8_t uart_putchar(device_handle device, char ch)
{
    uart_device_info *device_info = (uart_device_info *)device;
    if((!device) || (!device_info->init))
       return 0;

    if (device_info->uart_index == BAT485_UART_INDEX) {
        USARTx_SendByte(BAT_485, ch);
    } else if(device_info->uart_index == GW485_UART_INDEX) {
        USARTx_SendByte(GW_485, ch);
    }

    return 1;
}

/**
  * @brief  bat485ʹ.
  * @param
  * @retval
  */
void bat485_tx_enabla(void)
{
    GPIO_WriteBit(GPIO_BAT_485_RDE, Pin_BAT_485_RDE, write);
}

/**
  * @brief  bat485͹ر.
  * @param
  * @retval
  */
void bat485_tx_disenabla(void)
{
    GPIO_WriteBit(GPIO_BAT_485_RDE, Pin_BAT_485_RDE, read);
}

/**
  * @brief  gw485ʹ.
  * @param
  * @retval
  */
void gw485_tx_enabla(void)
{
    GPIO_WriteBit(GPIO_GW_485_RDE, Pin_GW_485_RDE, write);
}

/**
  * @brief  gw485͹ر.
  * @param
  * @retval
  */
void gw485_tx_disenabla(void)
{
    GPIO_WriteBit(GPIO_GW_485_RDE, Pin_GW_485_RDE, read);
}

/**
  * @brief  ַ.
  * @param  uart_index ں
  * @param  str ַַ
  * @retval None
  */
void uart_sendstr(device_handle device,char *str)
{
#if rs485_send_enable
  if(device == g_bat485_uart3_handle){
      bat485_tx_enabla();
  }
  else if (device == g_gw485_uart4_handle) {
      gw485_tx_enabla();
  }
#endif

    while ((*str) != (char )0u) {
        if (*str == ASCII_CHAR_LINE_FEED){
            uart_putchar(device, (ASCII_CHAR_CARRIAGE_RETURN));
            uart_putchar(device, ASCII_CHAR_LINE_FEED);
            str++;
        }else{
            uart_putchar(device, *str++);
        }
    }

#if rs485_send_enable
   if(device == g_bat485_uart3_handle){
       bat485_tx_disenabla();
   }
   else if (device == g_gw485_uart4_handle) {
       gw485_tx_disenabla();
   }
#endif
}



/**
  * @brief  ڶֽڷ.
  * @param  device ھ
  * @param  data 
  * @param  len  ݳ
  * @retval None
  */
void uart_dev_write(device_handle device, void *data, int len)
{
#if rs485_send_enable
  if(device == g_bat485_uart3_handle){
      bat485_tx_enabla();
  }
  else if (device == g_gw485_uart4_handle) {
      gw485_tx_enabla();
  }
#endif

   for (int i = 0; i < len; i++) {
//       uart_putchar(device, ((u_int8_t *)data)[i]);
       uart_putchar(device, ((char *)data)[i]);
   }

   Delay_Ms(1);

#if rs485_send_enable
   if(device == g_bat485_uart3_handle){
       bat485_tx_disenabla();
   }
   else if (device == g_gw485_uart4_handle) {
       gw485_tx_disenabla();
   }
#endif
}

/**
  * @brief  жϴ豸ѭbuffǷ.
  * @param  device ھ
  * @retval 0    1
  */
int uart_dev_char_present(device_handle device)
{
  uart_device_info *device_info = (uart_device_info *)device;

  if((!device) || (!device_info->init))
    return 0;

  return !RingQueueEmpty(&device_info->uart_ring_queue);
}

/**
  * @brief  Ӵ豸ѭbuffȡһ.
  * @param  device ھ
  * @retval ȡַ
  */
char uart_dev_in_char(device_handle device)
{
  uart_device_info *device_info = (uart_device_info *)device;
  char c = 0;

  if (uart_dev_char_present(device))
      OutRingQueue(&device_info->uart_ring_queue, (u_int8_t*)&c);
  return c;
}

/**
  * @brief  жѭbuffǷ.
  * @param  ring_queue ѭbuffַ
  * @retval 0    1
  */
int ring_queue_dev_char_present(RingQueue *ring_queue)
{
    return !RingQueueEmpty(ring_queue);
}

/**
  * @brief  ѭbuffȡһ.
  * @param  ring_queue ѭbuffַ
  * @retval ȡַ
  */
char ring_queue_dev_in_char(RingQueue *ring_queue)
{
  char c = 0;
  if (ring_queue_dev_char_present(ring_queue))
      OutRingQueue(ring_queue, (u_int8_t*)&c);
  return c;
}

/**
  * @brief  鿴ѭbuffеݸ.
  * @param  ring_queue ѭbuffַ
  * @retval ѭbuffеĸ
  */
int ring_queue_length(device_handle device)
{
    uart_device_info *device_info = (uart_device_info *)device;
    return RingQueueLength(&device_info->uart_ring_queue);
}








